"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || (function () {
    var ownKeys = function(o) {
        ownKeys = Object.getOwnPropertyNames || function (o) {
            var ar = [];
            for (var k in o) if (Object.prototype.hasOwnProperty.call(o, k)) ar[ar.length] = k;
            return ar;
        };
        return ownKeys(o);
    };
    return function (mod) {
        if (mod && mod.__esModule) return mod;
        var result = {};
        if (mod != null) for (var k = ownKeys(mod), i = 0; i < k.length; i++) if (k[i] !== "default") __createBinding(result, mod, k[i]);
        __setModuleDefault(result, mod);
        return result;
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
exports.RustNotionService = void 0;
// Use dynamic import with try-catch to handle module loading
let notionParallel = null;
try {
    // When the compiled Rust library exists, load it
    notionParallel = require("../rust/notion-parallel/index.node");
}
catch (error) {
    console.warn("Rust bindings for notion-parallel not found. Using TypeScript fallback implementation.");
}
/**
 * A wrapper class that provides access to the Rust implementation
 * of parallel processing for Notion API, with a fallback to the
 * TypeScript implementation if the Rust module is not available.
 */
class RustNotionService {
    reporter;
    notionApiKey;
    parallelLimit;
    enableCaching;
    cache;
    rustInstance = null;
    constructor(options) {
        this.reporter = options.reporter;
        this.notionApiKey = options.notionApiKey;
        this.parallelLimit = options.parallelLimit || 5;
        this.enableCaching = options.enableCaching !== false;
        this.cache = new Map();
        // Initialize Rust instance if available
        if (notionParallel) {
            try {
                this.rustInstance = new notionParallel.NotionParallel(this.notionApiKey, this.parallelLimit);
                this.reporter.info(`[RUST] Initialized Rust-based parallel processing with limit: ${this.parallelLimit}`);
            }
            catch (error) {
                this.reporter.warn(`[RUST] Failed to initialize Rust implementation: ${error}`);
                this.rustInstance = null;
            }
        }
    }
    /**
     * Get blocks for a single page, with recursive fetching of child blocks
     */
    async getPageBlocks(pageId) {
        const cacheKey = `page-blocks-${pageId}`;
        if (this.enableCaching && this.cache.has(cacheKey)) {
            this.reporter.info(`[CACHE] Using cached page blocks for ${pageId}`);
            return this.cache.get(cacheKey);
        }
        // Currently we don't have a direct Rust counterpart for a single page
        // So we use the multiple pages method with a single ID
        const results = await this.getMultiplePagesBlocks([pageId]);
        const blocks = results[pageId] || [];
        if (this.enableCaching) {
            this.cache.set(cacheKey, blocks);
        }
        return blocks;
    }
    /**
     * Get blocks for multiple pages in parallel, using Rust implementation if available
     */
    async getMultiplePagesBlocks(pageIds) {
        this.reporter.info(`[NOTION] Fetching blocks for ${pageIds.length} pages in parallel (limit: ${this.parallelLimit})`);
        // If Rust implementation is available and initialized, use it
        if (this.rustInstance) {
            try {
                this.reporter.info(`[RUST] Using Rust implementation for parallel processing`);
                const results = await this.rustInstance.getMultiplePagesBlocks(pageIds);
                return results;
            }
            catch (error) {
                this.reporter.warn(`[RUST] Error using Rust implementation, falling back to TypeScript: ${error}`);
                // Fall back to TypeScript implementation
            }
        }
        // Fallback to TypeScript implementation
        // Import the TypeScript implementation dynamically
        const { NotionService } = await Promise.resolve().then(() => __importStar(require("./api/service/notionService")));
        // Create a NotionService instance with the same configuration
        const tsService = new NotionService({
            reporter: this.reporter,
            parallelLimit: this.parallelLimit,
            enableCaching: this.enableCaching,
        });
        return tsService.getMultiplePagesBlocks(pageIds);
    }
    /**
     * Set the parallel processing limit
     */
    setParallelLimit(limit) {
        this.reporter.info(`[NOTION] Updated parallel request limit to ${limit}`);
        this.parallelLimit = limit;
        // Update the Rust instance if available
        if (this.rustInstance) {
            this.rustInstance.setParallelLimit(limit);
        }
    }
}
exports.RustNotionService = RustNotionService;
